/* Tabela TIPOOPERANA: Cadastro de Tipo de Operao para gerar anlises na entrada de notas*/
/* Descrio: A trigger insere registros na tabela LOGOPERA a partir de alteraes na tabela TIPOOPERANA */
/* Objetivo: Gerar um log que vai registrar as alteraes na tabela TIPOOPERANA,
             utilizando a tabela Geral de Logs de Operaes */

IF EXISTS(SELECT *
          FROM SYSOBJECTS O, SYSUSERS U
          WHERE O.NAME = 'LOG_QPR_TIPOOPERANA' AND
                TYPE   = 'TR'              AND
                U.UID  = O.UID             AND
                U.NAME = USER)

DROP TRIGGER LOG_QPR_TIPOOPERANA
GO
CREATE TRIGGER LOG_QPR_TIPOOPERANA
ON TIPOOPERANA
FOR UPDATE,INSERT, DELETE
AS
BEGIN
  DECLARE @V_USUARIO             VARCHAR(40);
  DECLARE @V_TIPO                VARCHAR(1);
  DECLARE @V_TPMOVI              VARCHAR(1);
  DECLARE @V_ITPESQ              VARCHAR(40);
  DECLARE @V_DESC                VARCHAR(500);
  DECLARE @V_CDFILLOGOPER        VARCHAR(4);
  DECLARE @V_DSREMOTEADDR        VARCHAR(15);
  DECLARE @V_DSREMOTEHOST        VARCHAR(15);
  DECLARE @V_DSSERVERRDM         VARCHAR(20);
  DECLARE @V_DSSERVMETHOD        VARCHAR(30);
  DECLARE @V_DSREMOTEOPER        VARCHAR(12);
  DECLARE @V_DTOPER              DATETIME;

  DECLARE @V_NRSEQTPOPANA_NEW    NUMERIC;
  DECLARE @V_CDFILIAL_NEW        VARCHAR(04);
  DECLARE @V_IDENTRSAIDOP_NEW    VARCHAR(01);
  DECLARE @V_CDTIPOOPER_NEW      VARCHAR(02);

  DECLARE @V_NRSEQTPOPANA_OLD    NUMERIC;
  DECLARE @V_CDFILIAL_OLD        VARCHAR(04);
  DECLARE @V_IDENTRSAIDOP_OLD    VARCHAR(01);
  DECLARE @V_CDTIPOOPER_OLD      VARCHAR(02);

   SET @V_TIPO = '0';
   SET @V_DESC = '';
   SET @V_CDFILLOGOPER = '';
   SET @V_USUARIO = '-';


   DECLARE C_CURSOR_TPA CURSOR FOR
      SELECT
         'D' IDCOMANDO,
         NULL, NULL, NULL, NULL,
         NRSEQTPOPANA,
         CDFILIAL,
         IDENTRSAIDOP,
         CDTIPOOPER
      FROM
         DELETED
      WHERE
         0 = (SELECT COUNT(*) FROM INSERTED)

      UNION ALL

      SELECT
         'U' IDCOMANDO,
         INSERTED.NRSEQTPOPANA,
         INSERTED.CDFILIAL,
         INSERTED.IDENTRSAIDOP,
         INSERTED.CDTIPOOPER,

         DELETED.NRSEQTPOPANA,
         DELETED.CDFILIAL,
         DELETED.IDENTRSAIDOP,
         DELETED.CDTIPOOPER
      FROM
         INSERTED,
         DELETED
      WHERE
         INSERTED.NRSEQTPOPANA = DELETED.NRSEQTPOPANA

      UNION ALL

      SELECT
         'I' IDCOMANDO,
         INSERTED.NRSEQTPOPANA,
         INSERTED.CDFILIAL,
         INSERTED.IDENTRSAIDOP,
         INSERTED.CDTIPOOPER,

         NULL, NULL, NULL, NULL
      FROM
         INSERTED
      WHERE
         0 = (SELECT COUNT(*) FROM DELETED)

   OPEN C_CURSOR_TPA;
   FETCH NEXT FROM C_CURSOR_TPA   INTO @V_TIPO,
                                       @V_NRSEQTPOPANA_NEW, @V_CDFILIAL_NEW, @V_IDENTRSAIDOP_NEW, @V_CDTIPOOPER_NEW,
                                       @V_NRSEQTPOPANA_OLD, @V_CDFILIAL_OLD, @V_IDENTRSAIDOP_OLD,@V_CDTIPOOPER_OLD;

   WHILE (@@FETCH_STATUS = 0) BEGIN

      IF @V_TIPO = 'D' BEGIN
         IF @V_NRSEQTPOPANA_OLD IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + 'NrSeq=' + CONVERT(VARCHAR, @V_NRSEQTPOPANA_OLD);
         END;

         IF @V_CDFILIAL_OLD IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + ' Filial=' + @V_CDFILIAL_OLD;
         END;

         IF @V_IDENTRSAIDOP_OLD IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + ' E/S=' + @V_IDENTRSAIDOP_OLD;
         END;

         IF @V_CDTIPOOPER_OLD IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + ' TpOper=' + @V_CDTIPOOPER_OLD;
         END;

         SET @V_TPMOVI       = '3';
         SET @V_ITPESQ       = @V_CDFILIAL_OLD+@V_IDENTRSAIDOP_OLD+@V_CDTIPOOPER_OLD;
         SET @V_CDFILLOGOPER = @V_CDFILIAL_OLD;
      END
      ELSE BEGIN
         IF @V_TIPO = 'U' BEGIN
            IF (@V_NRSEQTPOPANA_OLD <> @V_NRSEQTPOPANA_NEW) OR
               (@V_CDFILIAL_OLD     <> @V_CDFILIAL_NEW)     OR
               (@V_IDENTRSAIDOP_OLD <> @V_IDENTRSAIDOP_NEW) OR
               (@V_CDTIPOOPER_OLD   <> @V_CDTIPOOPER_NEW)  BEGIN

               SET @V_TPMOVI       = '2';
               SET @V_ITPESQ       = @V_CDFILIAL_OLD+@V_IDENTRSAIDOP_OLD+@V_CDTIPOOPER_OLD;
               SET @V_CDFILLOGOPER = @V_CDFILIAL_OLD;
            END;
         END
         ELSE BEGIN
            SET @V_TPMOVI        = '1';
            SET @V_ITPESQ        = @V_CDFILIAL_NEW+@V_IDENTRSAIDOP_NEW+@V_CDTIPOOPER_NEW;
            SET @V_CDFILLOGOPER  = @V_CDFILIAL_NEW;
         END;

         IF ISNULL(CONVERT(VARCHAR, @V_NRSEQTPOPANA_NEW), '!') <> ISNULL(CONVERT(VARCHAR, @V_NRSEQTPOPANA_OLD), '!') BEGIN
            SET @V_DESC = @V_DESC + 'NrSeq=' + CONVERT(VARCHAR, @V_NRSEQTPOPANA_NEW);
         END;

         IF ISNULL(@V_CDFILIAL_NEW, '!') <> ISNULL(@V_CDFILIAL_OLD, '!') BEGIN
            SET @V_DESC = @V_DESC + ' Filial=' + @V_CDFILIAL_NEW;
         END;

         IF ISNULL(@V_IDENTRSAIDOP_NEW, '!') <> ISNULL(@V_IDENTRSAIDOP_OLD, '!') BEGIN
            SET @V_DESC = @V_DESC + ' E/S=' + @V_IDENTRSAIDOP_NEW;
         END;

         IF ISNULL(@V_CDTIPOOPER_NEW, '!') <> ISNULL(@V_CDTIPOOPER_OLD, '!') BEGIN
            SET @V_DESC = @V_DESC + ' TpOper=' + @V_CDTIPOOPER_NEW;
         END;
      END;

      IF @V_TPMOVI <> '0' BEGIN
         EXEC NMUSUARIO  @V_USUARIO;
         EXEC WEBTUNNING @V_DSREMOTEADDR,
                         @V_DSREMOTEHOST,
                         @V_DSSERVERRDM,
                         @V_DSSERVMETHOD,
                         @V_DSREMOTEOPER;
         IF NOT @V_DESC IS NULL BEGIN

            SET @V_DTOPER = GETDATE();

            INSERT INTO LOGOPERA
               (NMTABELA,
                DTOPERACAO,
                NMRESPOPER,
                IDTIPOOPER,
                CDPESQITEM,
                DSOPERACAO,
                CDFILLOGOPER,
                DSRDMUSUOPER,
                DSENDUSUOPER,
                DSNMUSUOPER,
                DSMETUSUOPER,
                CDORESPOPER)
            VALUES
               ('TIPOOPERANA',
                @V_DTOPER,
                @V_USUARIO,
                @V_TPMOVI,
                @V_ITPESQ,
                SUBSTRING(@V_DESC,1,300),
                @V_CDFILLOGOPER,
                @V_DSSERVERRDM,
                @V_DSREMOTEADDR,
                @V_DSREMOTEHOST,
                @V_DSSERVMETHOD,
                @V_DSREMOTEOPER);

         END;
      END;
      FETCH NEXT FROM C_CURSOR_TPA INTO @V_TIPO,
                                        @V_NRSEQTPOPANA_NEW, @V_CDFILIAL_NEW, @V_IDENTRSAIDOP_NEW, @V_CDTIPOOPER_NEW,
                                        @V_NRSEQTPOPANA_OLD, @V_CDFILIAL_OLD, @V_IDENTRSAIDOP_OLD,@V_CDTIPOOPER_OLD;
   END;
   CLOSE C_CURSOR_TPA;
   DEALLOCATE C_CURSOR_TPA;
END;
GO


-- VERSAO --
EXEC ATUVEROBJ 'LOG_QPR_TIPOOPERANA','6.05.XXX';
GO

